use telers::{errors::SessionErrorKind as ErrorKind, event::EventReturn, types::ChatMember, Bot};

use crate::types::structs::message_sender::MessageSender;
use std::future::Future;

use super::{admin_check::is_admin, demote::demote_user, get_all_admins::get_all_admins};

const DEMOTE_FAILURE_MESSAGE: &str = "Команда не может быть выполнена: \
    не удалось удалить административные привилегии пользователя.";

pub async fn try_restrict<F, R>(
    future_callback: F,
    demote_args: (&Bot, i64, i64),
    failure_sender: MessageSender,
) -> Result<(), EventReturn>
where
    R: Future<Output = Result<bool, ErrorKind>>,
    F: Copy + Fn() -> R,
{
    let (bot, user_id, chat_id): (&Bot, i64, i64) = demote_args;

    let admins: Vec<ChatMember> = get_all_admins(bot, chat_id).await.unwrap();

    if is_admin(&admins, user_id) {
        MessageSender::builder(chat_id)
            .text("Нельзя применить эту команду на пользователе/бота, имеющего привелегии администратора.")
            .build()
            .send(bot)
            .await
            .unwrap();
        return Err(EventReturn::Cancel);
    }

    if future_callback().await.is_err() {
        if demote_user(bot, user_id, chat_id).await.is_err() {
            MessageSender::builder(chat_id)
                .text(DEMOTE_FAILURE_MESSAGE)
                .build()
                .send(bot)
                .await
                .unwrap();
            return Err(EventReturn::Cancel);
        }

        if future_callback().await.is_err() {
            failure_sender.send(bot).await.unwrap();
            return Err(EventReturn::Cancel);
        }
    }

    Ok(())
}
